import { Server } from 'socket.io'
import { ExtendedCharacter, TSocket } from '../../utilities/types'
import ZoneRepository from '../../repositories/zoneRepository'
import ZoneManager from '../../managers/zoneManager'
import { Character, Zone } from '@prisma/client'
import logger from '../../utilities/logger'

interface IPayload {
  zoneId: number
}

interface IResponse {
  zone: Zone
  characters: Character[]
}

/**
 * Handle character zone request event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
  socket.on('zone:characterJoin', async (data: IPayload, callback: (response: IResponse) => void) => {
    try {
      console.log(`---User ${socket.character?.id} has requested zone.`)

      if (!socket.character) return

      if (!data.zoneId) {
        console.log(`---Zone id not provided.`)
        return
      }

      const zone = await ZoneRepository.getById(data.zoneId)

      if (!zone) {
        console.log(`---Zone not found.`)
        return
      }

      if (socket.character?.zoneId) {
        socket.leave(socket.character.zoneId.toString())
        io.to(socket.character.zoneId.toString()).emit('zone:character:leave', socket.character)
      }

      socket.join(zone.id.toString())

      // let other clients know of new character
      io.to(zone.id.toString()).emit('zone:characterJoin', socket.character)

      // add character to zone manager
      ZoneManager.addCharacterToZone(zone.id, socket.character as Character)

      // send over zone and characters to socket
      callback({ zone, characters: ZoneManager.getCharactersInZone(zone.id) })
    } catch (error: any) {
      logger.error(`Error requesting zone: ${error.message}`)
      socket.disconnect()
    }
  })
}
