import { Server } from 'socket.io'
import { TSocket, ExtendedCharacter } from './types'

export class SocketEmitter {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public emitMoveError(message: string): void {
    this.socket.emit('character:moveError', message)
  }

  public emitCharacterMove(character: ExtendedCharacter): void {
    this.io.in(character.zoneId.toString()).emit('character:move', character)
  }

  public emitCharacterLeave(character: ExtendedCharacter, zoneId: number): void {
    this.io.to(zoneId.toString()).emit('zone:character:leave', character)
  }

  public emitCharacterJoin(character: ExtendedCharacter): void {
    this.io.to(character.zoneId.toString()).emit('zone:character:join', character)
  }

  public emitCharacterDataUpdated(character: ExtendedCharacter): void {
    this.socket.emit('character:dataUpdated', character)
  }
}
